@@ -40,9 +40,7 @@
 #include <libsn/sn.h>
 #endif
 
-#ifdef HAVE_XSYNC
 #include <X11/extensions/sync.h>
-#endif
 
 typedef struct _MetaKeyBinding MetaKeyBinding;
 typedef struct _MetaStack      MetaStack;
@@ -204,10 +202,8 @@ struct _MetaDisplay
   int         xkb_base_event_type;
   guint32     last_bell_time;
 #endif
-#ifdef HAVE_XSYNC
   /* alarm monitoring client's _NET_WM_SYNC_REQUEST_COUNTER */
   XSyncAlarm  grab_sync_request_alarm;
-#endif
   int	      grab_resize_timeout_id;
 
   /* Keybindings stuff */
@@ -256,14 +252,11 @@ struct _MetaDisplay
 #ifdef HAVE_STARTUP_NOTIFICATION
   SnDisplay *sn_display;
 #endif
-#ifdef HAVE_XSYNC
+
   int xsync_event_base;
   int xsync_error_base;
-#endif
-#ifdef HAVE_SHAPE
   int shape_event_base;
   int shape_error_base;
-#endif
 #ifdef HAVE_RENDER
   int render_event_base;
   int render_error_base;
@@ -271,25 +264,17 @@ struct _MetaDisplay
 #ifdef HAVE_COMPOSITE_EXTENSIONS
   int composite_event_base;
   int composite_error_base;
-  int composite_major_version;
-  int composite_minor_version;
   int damage_event_base;
   int damage_error_base;
   int xfixes_event_base;
   int xfixes_error_base;
 #endif
-#ifdef HAVE_XSYNC
+
   unsigned int have_xsync : 1;
 #define META_DISPLAY_HAS_XSYNC(display) ((display)->have_xsync)
-#else
-#define META_DISPLAY_HAS_XSYNC(display) FALSE
-#endif
-#ifdef HAVE_SHAPE
   unsigned int have_shape : 1;
 #define META_DISPLAY_HAS_SHAPE(display) ((display)->have_shape)
-#else
-#define META_DISPLAY_HAS_SHAPE(display) FALSE
-#endif
+
 #ifdef HAVE_RENDER
   unsigned int have_render : 1;
 #define META_DISPLAY_HAS_RENDER(display) ((display)->have_render)
