@@ -76,7 +76,6 @@ static int num_workspaces = 4;
 static GDesktopTitlebarAction action_double_click_titlebar = G_DESKTOP_TITLEBAR_ACTION_TOGGLE_MAXIMIZE;
 static GDesktopTitlebarAction action_middle_click_titlebar = G_DESKTOP_TITLEBAR_ACTION_LOWER;
 static GDesktopTitlebarAction action_right_click_titlebar = G_DESKTOP_TITLEBAR_ACTION_MENU;
-static gboolean application_based = FALSE;
 static gboolean disable_workarounds = FALSE;
 static gboolean auto_raise = FALSE;
 static gboolean auto_raise_delay = 500;
@@ -87,10 +86,11 @@ static gboolean gnome_accessibility = FALSE;
 static gboolean gnome_animations = TRUE;
 static char *cursor_theme = NULL;
 static int   cursor_size = 24;
-static gboolean compositing_manager = FALSE;
+static gboolean compositing_manager = TRUE;
 static gboolean resize_with_right_button = FALSE;
 static gboolean edge_tiling = FALSE;
 static gboolean force_fullscreen = TRUE;
+static gboolean alt_tab_thumbnails = FALSE;
 
 static GDesktopVisualBellType visual_bell_type = G_DESKTOP_VISUAL_BELL_FULLSCREEN_FLASH;
 static MetaButtonLayout button_layout;
@@ -275,14 +275,6 @@ static MetaBoolPreference preferences_bool[] =
       &use_system_font,
       TRUE,
     },
-    {
-      { "application-based",
-        SCHEMA_GENERAL,
-        META_PREF_APPLICATION_BASED,
-      },
-      NULL, /* feature is known but disabled */
-      FALSE,
-    },
     {
       { "disable-workarounds",
         SCHEMA_GENERAL,
@@ -363,6 +355,14 @@ static MetaBoolPreference preferences_bool[] =
       &edge_tiling,
       FALSE,
     },
+    {
+      { "alt-tab-thumbnails",
+        SCHEMA_METACITY,
+        META_PREF_ALT_TAB_THUMBNAILS,
+      },
+      &alt_tab_thumbnails,
+      FALSE,
+    },
     { { NULL, 0, 0 }, NULL, FALSE },
   };
 
@@ -378,7 +378,7 @@ static MetaStringPreference preferences_string[] =
     },
     {
       { "theme",
-        SCHEMA_GENERAL,
+        SCHEMA_METACITY,
         META_PREF_THEME,
       },
       theme_name_handler,
@@ -924,10 +924,7 @@ meta_prefs_get_attach_modal_dialogs (void)
 gboolean
 meta_prefs_get_raise_on_click (void)
 {
-  /* Force raise_on_click on for click-to-focus, as requested by Havoc
-   * in #326156.
-   */
-  return raise_on_click || focus_mode == G_DESKTOP_FOCUS_MODE_CLICK;
+  return raise_on_click;
 }
 
 const char*
@@ -1002,15 +999,16 @@ theme_name_handler (GVariant *value,
   *result = NULL; /* ignored */
   string_value = g_variant_get_string (value, NULL);
 
-  if (!string_value || !*string_value)
-    return FALSE;
-
   if (g_strcmp0 (current_theme, string_value) != 0)
     {
       if (current_theme)
         g_free (current_theme);
 
-      current_theme = g_strdup (string_value);
+      if (!string_value || !*string_value)
+        current_theme = NULL;
+      else
+        current_theme = g_strdup (string_value);
+
       queue_changed (META_PREF_THEME);
     }
 
@@ -1332,14 +1330,6 @@ meta_prefs_get_num_workspaces (void)
   return num_workspaces;
 }
 
-gboolean
-meta_prefs_get_application_based (void)
-{
-  return FALSE; /* For now, we never want this to do anything */
-
-  return application_based;
-}
-
 gboolean
 meta_prefs_get_disable_workarounds (void)
 {
@@ -1377,9 +1367,6 @@ meta_preference_to_string (MetaPreference pref)
     case META_PREF_NUM_WORKSPACES:
       return "NUM_WORKSPACES";
 
-    case META_PREF_APPLICATION_BASED:
-      return "APPLICATION_BASED";
-
     case META_PREF_KEYBINDINGS:
       return "KEYBINDINGS";
 
@@ -1445,6 +1432,9 @@ meta_preference_to_string (MetaPreference pref)
 
     case META_PREF_PLACEMENT_MODE:
       return "PLACEMENT_MODE";
+
+    case META_PREF_ALT_TAB_THUMBNAILS:
+      return "ALT_TAB_THUMBNAILS";
     }
 
   return "(unknown)";
@@ -1867,6 +1857,12 @@ meta_prefs_get_placement_mode (void)
   return placement_mode;
 }
 
+gboolean
+meta_prefs_get_alt_tab_thumbnails (void)
+{
+  return alt_tab_thumbnails;
+}
+
 void
 meta_prefs_set_compositing_manager (gboolean whether)
 {
