@@ -14,11 +14,11 @@
  * together.
  */
 
-/* 
+/*
  * Copyright (C) 2001, 2002, 2003 Red Hat, Inc.
  * Copyright (C) 2004, 2005 Elijah Newren
  * Copyright (C) 2009 Thomas Thurman
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation; either version 2 of the
@@ -28,7 +28,7 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
@@ -64,97 +64,138 @@ typedef void (* ReloadValueFunc) (MetaWindow    *window,
                                   MetaPropValue *value,
                                   gboolean       initial);
 
-typedef struct MetaWindowPropHooks
-{
+typedef enum {
+  NONE       = 0,
+  LOAD_INIT  = (1 << 0),
+  INIT_ONLY  = (1 << 1),
+  FORCE_INIT = (1 << 2),
+} MetaPropHookFlags;
+
+struct _MetaWindowPropHooks {
   Atom              property;
   MetaPropValueType type;
   ReloadValueFunc   reload_func;
-} MetaWindowPropHooks;
+  MetaPropHookFlags flags;
+};
 
+static void init_prop_value            (MetaWindow          *window,
+                                        MetaWindowPropHooks *hooks,
+                                        MetaPropValue       *value);
+static void reload_prop_value          (MetaWindow          *window,
+                                        MetaWindowPropHooks *hooks,
+                                        MetaPropValue       *value,
+                                        gboolean             initial);
 static MetaWindowPropHooks* find_hooks (MetaDisplay *display,
                                         Atom         property);
 
-
-void
-meta_window_reload_property (MetaWindow *window,
-                             Atom        property,
-                             gboolean    initial)
-{
-  meta_window_reload_properties (window, &property, 1, initial);
-}
-
-void
-meta_window_reload_properties (MetaWindow *window,
-                               const Atom *properties,
-                               int         n_properties,
-                               gboolean    initial)
-{
-  meta_window_reload_properties_from_xwindow (window,
-                                              window->xwindow,
-                                              properties,
-                                              n_properties,
-                                              initial);
-}
-
 void
 meta_window_reload_property_from_xwindow (MetaWindow *window,
                                           Window      xwindow,
                                           Atom        property,
                                           gboolean    initial)
 {
-  meta_window_reload_properties_from_xwindow (window, xwindow, &property, 1,
-                                              initial);
+  MetaPropValue value = { 0, };
+  MetaWindowPropHooks *hooks;
+
+  hooks = find_hooks (window->display, property);
+  if (!hooks)
+    return;
+
+  init_prop_value (window, hooks, &value);
+
+  meta_prop_get_values (window->display, xwindow,
+                        &value, 1);
+
+  reload_prop_value (window, hooks, &value,
+                     initial);
+
+  meta_prop_free_values (&value, 1);
 }
 
 void
-meta_window_reload_properties_from_xwindow (MetaWindow *window,
-                                            Window      xwindow,
-                                            const Atom *properties,
-                                            int         n_properties,
-                                            gboolean    initial)
+meta_window_reload_property (MetaWindow *window,
+                             Atom        property,
+                             gboolean    initial)
 {
-  int i;
+  meta_window_reload_property_from_xwindow (window,
+                                            window->xwindow,
+                                            property,
+                                            initial);
+}
+
+void
+meta_window_load_initial_properties (MetaWindow *window)
+{
+  int i, j;
   MetaPropValue *values;
+  int n_properties = 0;
 
-  g_return_if_fail (properties != NULL);
-  g_return_if_fail (n_properties > 0);
-  
-  values = g_new0 (MetaPropValue, n_properties);
+  values = g_new0 (MetaPropValue, window->display->n_prop_hooks);
 
-  for (i=0; i<n_properties; i++)
+  j = 0;
+  for (i = 0; i < window->display->n_prop_hooks; i++)
     {
-      MetaWindowPropHooks *hooks = find_hooks (window->display,
-                                               properties[i]);
-    
-      if (!hooks || hooks->type == META_PROP_VALUE_INVALID)
+      MetaWindowPropHooks *hooks = &window->display->prop_hooks_table[i];
+      if (hooks->flags & LOAD_INIT)
         {
-          values[i].type = META_PROP_VALUE_INVALID;
-          values[i].atom = None;
-        }
-      else
-        {
-          values[i].type = hooks->type;
-          values[i].atom = properties[i];
+          init_prop_value (window, hooks, &values[j]);
+          ++j;
         }
     }
-  
-  meta_prop_get_values (window->display, xwindow,
+  n_properties = j;
+
+  meta_prop_get_values (window->display, window->xwindow,
                         values, n_properties);
 
-  for (i=0; i<n_properties; i++)
+  j = 0;
+  for (i = 0; i < window->display->n_prop_hooks; i++)
     {
-      MetaWindowPropHooks *hooks = find_hooks (window->display,
-                                               properties[i]);
-
-      if (hooks && hooks->reload_func != NULL)
-        (* hooks->reload_func) (window, &values[i], initial);
+      MetaWindowPropHooks *hooks = &window->display->prop_hooks_table[i];
+      if (hooks->flags & LOAD_INIT)
+        {
+          /* If we didn't actually manage to load anything then we don't need
+           * to call the reload function; this is different from a notification
+           * where disappearance of a previously present value is significant.
+           */
+          if (values[j].type != META_PROP_VALUE_INVALID || hooks->flags & FORCE_INIT)
+            reload_prop_value (window, hooks, &values[j], TRUE);
+          ++j;
+        }
     }
 
   meta_prop_free_values (values, n_properties);
-  
+
   g_free (values);
 }
 
+/* Fill in the MetaPropValue used to get the value of "property" */
+static void
+init_prop_value (MetaWindow          *window,
+                 MetaWindowPropHooks *hooks,
+                 MetaPropValue       *value)
+{
+  if (!hooks || hooks->type == META_PROP_VALUE_INVALID)
+    {
+      value->type = META_PROP_VALUE_INVALID;
+      value->atom = None;
+    }
+  else
+    {
+      value->type = hooks->type;
+      value->atom = hooks->property;
+    }
+}
+
+static void
+reload_prop_value (MetaWindow          *window,
+                   MetaWindowPropHooks *hooks,
+                   MetaPropValue       *value,
+                   gboolean             initial)
+{
+  if (hooks && hooks->reload_func != NULL)
+    (* hooks->reload_func) (window, value, initial);
+}
+
 static void
 reload_wm_client_machine (MetaWindow    *window,
                           MetaPropValue *value,
@@ -162,7 +203,7 @@ reload_wm_client_machine (MetaWindow    *window,
 {
   g_free (window->wm_client_machine);
   window->wm_client_machine = NULL;
-  
+
   if (value->type != META_PROP_VALUE_INVALID)
     window->wm_client_machine = g_strdup (value->v.str);
 
@@ -184,7 +225,62 @@ reload_net_wm_window_type (MetaWindow    *window,
                            MetaPropValue *value,
                            gboolean       initial)
 {
-  meta_window_update_net_wm_type (window);
+  int n_atoms;
+  Atom *atoms;
+  int i;
+
+  window->type_atom = None;
+  n_atoms = 0;
+  atoms = NULL;
+
+  meta_prop_get_atom_list (window->display, window->xwindow,
+                           window->display->atom__NET_WM_WINDOW_TYPE,
+                           &atoms, &n_atoms);
+
+  i = 0;
+  while (i < n_atoms)
+    {
+      /* We break as soon as we find one we recognize,
+       * supposed to prefer those near the front of the list
+       */
+      if (atoms[i] == window->display->atom__NET_WM_WINDOW_TYPE_DESKTOP ||
+          atoms[i] == window->display->atom__NET_WM_WINDOW_TYPE_DOCK ||
+          atoms[i] == window->display->atom__NET_WM_WINDOW_TYPE_TOOLBAR ||
+          atoms[i] == window->display->atom__NET_WM_WINDOW_TYPE_MENU ||
+          atoms[i] == window->display->atom__NET_WM_WINDOW_TYPE_DIALOG ||
+          atoms[i] == window->display->atom__NET_WM_WINDOW_TYPE_NORMAL ||
+          atoms[i] == window->display->atom__NET_WM_WINDOW_TYPE_UTILITY ||
+          atoms[i] == window->display->atom__NET_WM_WINDOW_TYPE_SPLASH)
+        {
+          window->type_atom = atoms[i];
+          break;
+        }
+
+      ++i;
+    }
+
+  meta_XFree (atoms);
+
+  if (meta_is_verbose ())
+    {
+      char *str;
+
+      str = NULL;
+      if (window->type_atom != None)
+        {
+          meta_error_trap_push (window->display);
+          str = XGetAtomName (window->display->xdisplay, window->type_atom);
+          meta_error_trap_pop (window->display, TRUE);
+        }
+
+      meta_verbose ("Window %s type atom %s\n", window->desc,
+                    str ? str : "(none)");
+
+      if (str)
+        meta_XFree (str);
+    }
+
+  meta_window_recalc_window_type (window);
 }
 
 static void
@@ -213,6 +309,68 @@ reload_kwm_win_icon (MetaWindow    *window,
   reload_icon (window, window->display->atom__KWM_WIN_ICON);
 }
 
+static gboolean
+gtk_border_equal (GtkBorder *a,
+                  GtkBorder *b)
+{
+  return (a->left == b->left &&
+          a->right == b->right &&
+          a->top == b->top &&
+          a->bottom == b->bottom);
+}
+
+static void
+meta_window_set_custom_frame_extents (MetaWindow *window,
+                                      GtkBorder  *extents)
+{
+  if (extents)
+    {
+      if (window->has_custom_frame_extents && gtk_border_equal (&window->custom_frame_extents, extents))
+        return;
+
+      window->has_custom_frame_extents = TRUE;
+      window->custom_frame_extents = *extents;
+    }
+  else
+    {
+      if (!window->has_custom_frame_extents)
+        return;
+
+      window->has_custom_frame_extents = FALSE;
+      memset (&window->custom_frame_extents, 0, sizeof (window->custom_frame_extents));
+    }
+
+  meta_window_queue (window, META_QUEUE_MOVE_RESIZE);
+}
+
+static void
+reload_gtk_frame_extents (MetaWindow    *window,
+                          MetaPropValue *value,
+                          gboolean       initial)
+{
+  if (value->type != META_PROP_VALUE_INVALID)
+    {
+      if (value->v.cardinal_list.n_cardinals != 4)
+        {
+          meta_verbose ("_GTK_FRAME_EXTENTS on %s has %d values instead of 4\n",
+                        window->desc, value->v.cardinal_list.n_cardinals);
+        }
+      else
+        {
+          GtkBorder extents;
+          extents.left   = (int)value->v.cardinal_list.cardinals[0];
+          extents.right  = (int)value->v.cardinal_list.cardinals[1];
+          extents.top    = (int)value->v.cardinal_list.cardinals[2];
+          extents.bottom = (int)value->v.cardinal_list.cardinals[3];
+          meta_window_set_custom_frame_extents (window, &extents);
+        }
+    }
+  else
+    {
+      meta_window_set_custom_frame_extents (window, NULL);
+    }
+}
+
 static void
 reload_struts (MetaWindow    *window,
                MetaPropValue *value,
@@ -226,7 +384,9 @@ reload_wm_window_role (MetaWindow    *window,
                        MetaPropValue *value,
                        gboolean       initial)
 {
-  meta_window_update_role (window);
+  g_clear_pointer (&window->role, g_free);
+  if (value->type != META_PROP_VALUE_INVALID)
+    window->role = g_strdup (value->v.str);
 }
 
 static void
@@ -237,7 +397,7 @@ reload_net_wm_pid (MetaWindow    *window,
   if (value->type != META_PROP_VALUE_INVALID)
     {
       gulong cardinal = (int) value->v.cardinal;
-      
+
       if (cardinal <= 0)
         meta_warning (_("Application set a bogus _NET_WM_PID %lu\n"),
                       cardinal);
@@ -332,7 +492,7 @@ owner_of_process (pid_t process, uid_t *result)
 {
 #ifdef HAVE_GTOP
   glibtop_proc_uid process_details;
-      
+
   glibtop_get_proc_uid (&process_details, process);
 
   if (process_details.flags & (1L << GLIBTOP_PROC_UID_UID))
@@ -370,12 +530,12 @@ set_title_text (MetaWindow  *window,
 {
   char hostname[HOST_NAME_MAX + 1];
   gboolean modified = FALSE;
-  
+
   if (!target)
     return FALSE;
-  
+
   g_free (*target);
-  
+
   if (!title)
     *target = g_strdup ("");
   else if (g_utf8_strlen (title, MAX_TITLE_LENGTH + 1) > MAX_TITLE_LENGTH)
@@ -409,7 +569,7 @@ set_title_text (MetaWindow  *window,
       /* Assume a window with unknown ownership is ours (call it usufruct!) */
       gboolean window_owner_is_us =
               !window_owner_known || window_owner==getuid ();
-      
+
       if (window_owner_is_us)
         {
           /* we own it, so fall back to the simple case */
@@ -436,7 +596,7 @@ set_title_text (MetaWindow  *window,
                 {
                   found_name = pwd->pw_name;
                 }
-            
+
               if (found_name)
                 /* Translators: the title of a window owned by another user
                  * on this machine */
@@ -451,7 +611,7 @@ set_title_text (MetaWindow  *window,
             }
           /* either way we changed it */
           modified = TRUE;
-              
+
         }
     }
   else
@@ -479,8 +639,6 @@ static void
 set_window_title (MetaWindow *window,
                   const char *title)
 {
-  char *str;
- 
   gboolean modified =
     set_title_text (window,
                     window->using_net_wm_visible_name,
@@ -488,12 +646,9 @@ set_window_title (MetaWindow *window,
                     window->display->atom__NET_WM_VISIBLE_NAME,
                     &window->title);
   window->using_net_wm_visible_name = modified;
-  
-  /* strndup is a hack since GNU libc has broken %.10s */
-  str = g_strndup (window->title, 10);
+
   g_free (window->desc);
-  window->desc = g_strdup_printf ("0x%lx (%s)", window->xwindow, str);
-  g_free (str);
+  window->desc = g_strdup_printf ("0x%lx (%.30s)", window->xwindow, window->title);
 
   if (window->frame)
     meta_ui_set_frame_title (window->screen->ui,
@@ -534,7 +689,7 @@ reload_wm_name (MetaWindow    *window,
                     value->v.str);
       return;
     }
-  
+
   if (value->type != META_PROP_VALUE_INVALID)
     {
       set_window_title (window, value->v.str);
@@ -594,11 +749,11 @@ reload_wm_icon_name (MetaWindow    *window,
                     value->v.str);
       return;
     }
-  
+
   if (value->type != META_PROP_VALUE_INVALID)
     {
       set_icon_title (window, value->v.str);
-      
+
       meta_verbose ("Using WM_ICON_NAME for new title of %s: \"%s\"\n",
                     window->desc, window->title);
     }
@@ -675,6 +830,7 @@ reload_net_wm_state (MetaWindow    *window,
                 window->desc);
 
   meta_window_recalc_window_type (window);
+  meta_window_recalc_features (window);
 }
 
 static void
@@ -786,7 +942,7 @@ reload_mwm_hints (MetaWindow    *window,
     meta_verbose ("Functions flag unset\n");
 
   meta_window_recalc_features (window);
-  
+
   /* We do all this anyhow at the end of meta_window_new() */
   if (!window->constructing)
     {
@@ -794,7 +950,7 @@ reload_mwm_hints (MetaWindow    *window,
         meta_window_ensure_frame (window);
       else
         meta_window_destroy_frame (window);
-      
+
       meta_window_queue (window,
                          META_QUEUE_MOVE_RESIZE |
                          /* because ensure/destroy frame may unmap: */
@@ -816,7 +972,7 @@ reload_wm_class (MetaWindow    *window,
   window->res_name = NULL;
 
   if (value->type != META_PROP_VALUE_INVALID)
-    { 
+    {
       if (value->v.class_hint.res_name)
         window->res_name = g_strdup (value->v.class_hint.res_name);
 
@@ -852,32 +1008,32 @@ reload_net_startup_id (MetaWindow    *window,
 {
   guint32 timestamp = window->net_wm_user_time;
   MetaWorkspace *workspace = NULL;
-  
+
   g_free (window->startup_id);
-  
+
   if (value->type != META_PROP_VALUE_INVALID)
     window->startup_id = g_strdup (value->v.str);
   else
     window->startup_id = NULL;
-    
+
   /* Update timestamp and workspace on a running window */
   if (!window->constructing)
   {
-    window->initial_timestamp_set = 0;  
+    window->initial_timestamp_set = 0;
     window->initial_workspace_set = 0;
-    
+
     if (meta_screen_apply_startup_properties (window->screen, window))
       {
-  
+
         if (window->initial_timestamp_set)
           timestamp = window->initial_timestamp;
         if (window->initial_workspace_set)
           workspace = meta_screen_get_workspace_by_index (window->screen, window->initial_workspace);
-    
+
         meta_window_activate_with_workspace (window, timestamp, workspace);
       }
   }
-  
+
   meta_verbose ("New _NET_STARTUP_ID \"%s\" for %s\n",
                 window->startup_id ? window->startup_id : "unset",
                 window->desc);
@@ -890,13 +1046,11 @@ reload_update_counter (MetaWindow    *window,
 {
   if (value->type != META_PROP_VALUE_INVALID)
     {
-#ifdef HAVE_XSYNC
       XSyncCounter counter = value->v.xcounter;
 
       window->sync_request_counter = counter;
       meta_verbose ("Window has _NET_WM_SYNC_REQUEST_COUNTER 0x%lx\n",
                     window->sync_request_counter);
-#endif
     }
 }
 
@@ -957,7 +1111,7 @@ spew_size_hints_differences (const XSizeHints *old,
   if (FLAG_CHANGED (old, new, PWinGravity))
     meta_topic (META_DEBUG_GEOMETRY, "XSizeHints: PWinGravity now %s  (%d -> %d)\n",
                 FLAG_TOGGLED_ON (old, new, PWinGravity) ? "set" : "unset",
-                old->win_gravity, new->win_gravity);  
+                old->win_gravity, new->win_gravity);
 }
 
 void
@@ -1310,15 +1464,15 @@ reload_normal_hints (MetaWindow    *window,
   if (value->type != META_PROP_VALUE_INVALID)
     {
       XSizeHints old_hints;
-  
+
       meta_topic (META_DEBUG_GEOMETRY, "Updating WM_NORMAL_HINTS for %s\n", window->desc);
 
       old_hints = window->size_hints;
-  
+
       meta_set_normal_hints (window, value->v.size_hints.hints);
-      
+
       spew_size_hints_differences (&old_hints, &window->size_hints);
-      
+
       meta_window_recalc_features (window);
 
       if (!initial)
@@ -1332,12 +1486,12 @@ reload_wm_protocols (MetaWindow    *window,
                      gboolean       initial)
 {
   int i;
-  
+
   window->take_focus = FALSE;
   window->delete_window = FALSE;
   window->net_wm_ping = FALSE;
-  
-  if (value->type == META_PROP_VALUE_INVALID)    
+
+  if (value->type == META_PROP_VALUE_INVALID)
     return;
 
   i = 0;
@@ -1354,7 +1508,7 @@ reload_wm_protocols (MetaWindow    *window,
         window->net_wm_ping = TRUE;
       ++i;
     }
-  
+
   meta_verbose ("New _NET_STARTUP_ID \"%s\" for %s\n",
                 window->startup_id ? window->startup_id : "unset",
                 window->desc);
@@ -1366,20 +1520,20 @@ reload_wm_hints (MetaWindow    *window,
                  gboolean       initial)
 {
   Window old_group_leader;
-  
+
   old_group_leader = window->xgroup_leader;
-  
+
   /* Fill in defaults */
   window->input = TRUE;
   window->initially_iconic = FALSE;
   window->xgroup_leader = None;
   window->wm_hints_pixmap = None;
   window->wm_hints_mask = None;
-  
+
   if (value->type != META_PROP_VALUE_INVALID)
     {
       const XWMHints *hints = value->v.wm_hints;
-      
+
       if (hints->flags & InputHint)
         window->input = hints->input;
 
@@ -1394,7 +1548,7 @@ reload_wm_hints (MetaWindow    *window,
 
       if (hints->flags & IconMaskHint)
         window->wm_hints_mask = hints->icon_mask;
-      
+
       meta_verbose ("Read WM_HINTS input: %d iconic: %d group leader: 0x%lx pixmap: 0x%lx mask: 0x%lx\n",
                     window->input, window->initially_iconic,
                     window->xgroup_leader,
@@ -1406,7 +1560,7 @@ reload_wm_hints (MetaWindow    *window,
     {
       meta_verbose ("Window %s changed its group leader to 0x%lx\n",
                     window->desc, window->xgroup_leader);
-      
+
       meta_window_group_leader_changed (window);
     }
 
@@ -1423,13 +1577,13 @@ reload_transient_for (MetaWindow    *window,
                       gboolean       initial)
 {
   window->xtransient_for = None;
-  
+
   if (value->type != META_PROP_VALUE_INVALID)
     window->xtransient_for = value->v.xwindow;
 
   /* Make sure transient_for is valid */
   if (window->xtransient_for != None &&
-      meta_display_lookup_x_window (window->display, 
+      meta_display_lookup_x_window (window->display,
                                     window->xtransient_for) == NULL)
     {
       meta_warning (_("Invalid WM_TRANSIENT_FOR window 0x%lx specified "
@@ -1466,6 +1620,32 @@ reload_transient_for (MetaWindow    *window,
     meta_window_queue (window, META_QUEUE_MOVE_RESIZE);
 }
 
+static void
+reload_gtk_theme_variant (MetaWindow    *window,
+                          MetaPropValue *value,
+                          gboolean       initial)
+{
+  char *requested_variant = NULL;
+  char *current_variant = window->gtk_theme_variant;
+
+  if (value->type != META_PROP_VALUE_INVALID)
+    {
+      requested_variant = value->v.str;
+      meta_verbose ("Requested \"%s\" theme variant for window %s.\n",
+                    requested_variant, window->desc);
+    }
+
+  if (g_strcmp0 (requested_variant, current_variant) != 0)
+    {
+      g_free (current_variant);
+
+      window->gtk_theme_variant = g_strdup (requested_variant);
+
+      if (window->frame)
+        meta_ui_update_frame_style (window->screen->ui, window->frame->xwindow);
+    }
+}
+
 /**
  * Initialises the property hooks system.  Each row in the table named "hooks"
  * represents an action to take when a property is found on a newly-created
@@ -1484,35 +1664,183 @@ void
 meta_display_init_window_prop_hooks (MetaDisplay *display)
 {
   MetaWindowPropHooks hooks[] = {
-    { display->atom_WM_STATE,          META_PROP_VALUE_INVALID,  NULL },
-    { display->atom_WM_CLIENT_MACHINE, META_PROP_VALUE_STRING,   reload_wm_client_machine },
-    { display->atom__NET_WM_PID,       META_PROP_VALUE_CARDINAL, reload_net_wm_pid },
-    { display->atom__NET_WM_USER_TIME, META_PROP_VALUE_CARDINAL, reload_net_wm_user_time },
-    { display->atom__NET_WM_NAME,      META_PROP_VALUE_UTF8,     reload_net_wm_name },
-    { XA_WM_NAME,                      META_PROP_VALUE_TEXT_PROPERTY, reload_wm_name },
-    { display->atom__NET_WM_ICON,      META_PROP_VALUE_INVALID,  reload_net_wm_icon },
-    { display->atom__KWM_WIN_ICON,     META_PROP_VALUE_INVALID,  reload_kwm_win_icon },
-    { display->atom__NET_WM_ICON_NAME, META_PROP_VALUE_UTF8,     reload_net_wm_icon_name },
-    { XA_WM_ICON_NAME,                 META_PROP_VALUE_TEXT_PROPERTY, reload_wm_icon_name },
-    { display->atom__NET_WM_STATE,     META_PROP_VALUE_ATOM_LIST, reload_net_wm_state },
-    { display->atom__MOTIF_WM_HINTS,   META_PROP_VALUE_MOTIF_HINTS, reload_mwm_hints },
-    { display->atom__NET_WM_ICON_GEOMETRY, META_PROP_VALUE_INVALID, NULL },
-    { XA_WM_CLASS,                     META_PROP_VALUE_CLASS_HINT, reload_wm_class },
-    { display->atom_WM_CLIENT_LEADER,  META_PROP_VALUE_INVALID, complain_about_broken_client },
-    { display->atom_SM_CLIENT_ID,      META_PROP_VALUE_INVALID, complain_about_broken_client },
-    { display->atom_WM_WINDOW_ROLE,    META_PROP_VALUE_INVALID, reload_wm_window_role },
-    { display->atom__NET_WM_WINDOW_TYPE, META_PROP_VALUE_INVALID, reload_net_wm_window_type },
-    { display->atom__NET_WM_DESKTOP,   META_PROP_VALUE_CARDINAL, reload_net_wm_desktop },
-    { display->atom__NET_WM_STRUT,         META_PROP_VALUE_INVALID, reload_struts },
-    { display->atom__NET_WM_STRUT_PARTIAL, META_PROP_VALUE_INVALID, reload_struts },
-    { display->atom__NET_STARTUP_ID,  META_PROP_VALUE_UTF8,     reload_net_startup_id },
-    { display->atom__NET_WM_SYNC_REQUEST_COUNTER, META_PROP_VALUE_SYNC_COUNTER, reload_update_counter },
-    { XA_WM_NORMAL_HINTS,              META_PROP_VALUE_SIZE_HINTS, reload_normal_hints },
-    { display->atom_WM_PROTOCOLS,      META_PROP_VALUE_ATOM_LIST, reload_wm_protocols },
-    { XA_WM_HINTS,                     META_PROP_VALUE_WM_HINTS,  reload_wm_hints },
-    { XA_WM_TRANSIENT_FOR,             META_PROP_VALUE_WINDOW,    reload_transient_for },
-    { display->atom__NET_WM_USER_TIME_WINDOW, META_PROP_VALUE_WINDOW, reload_net_wm_user_time_window },
-    { 0 },
+    {
+      display->atom_WM_CLIENT_MACHINE,
+      META_PROP_VALUE_STRING,
+      reload_wm_client_machine,
+      LOAD_INIT
+    },
+    {
+      display->atom__NET_WM_NAME,
+      META_PROP_VALUE_UTF8,
+      reload_net_wm_name,
+      LOAD_INIT
+    },
+    {
+      XA_WM_CLASS,
+      META_PROP_VALUE_CLASS_HINT,
+      reload_wm_class,
+      LOAD_INIT
+    },
+    {
+      display->atom__NET_WM_PID,
+      META_PROP_VALUE_CARDINAL,
+      reload_net_wm_pid,
+      LOAD_INIT
+    },
+    {
+      XA_WM_NAME,
+      META_PROP_VALUE_TEXT_PROPERTY,
+      reload_wm_name,
+      LOAD_INIT
+    },
+    {
+      display->atom__NET_WM_ICON_NAME,
+      META_PROP_VALUE_UTF8,
+      reload_net_wm_icon_name,
+      LOAD_INIT
+    },
+    {
+      XA_WM_ICON_NAME,
+      META_PROP_VALUE_TEXT_PROPERTY,
+      reload_wm_icon_name,
+      LOAD_INIT
+    },
+    {
+      display->atom__NET_WM_DESKTOP,
+      META_PROP_VALUE_CARDINAL,
+      reload_net_wm_desktop,
+      LOAD_INIT | INIT_ONLY
+    },
+    {
+      display->atom__NET_STARTUP_ID,
+      META_PROP_VALUE_UTF8,
+      reload_net_startup_id,
+      LOAD_INIT
+    },
+    {
+      display->atom__NET_WM_SYNC_REQUEST_COUNTER,
+      META_PROP_VALUE_SYNC_COUNTER,
+      reload_update_counter,
+      LOAD_INIT
+    },
+    {
+      XA_WM_NORMAL_HINTS,
+      META_PROP_VALUE_SIZE_HINTS,
+      reload_normal_hints,
+      LOAD_INIT
+    },
+    {
+      display->atom_WM_PROTOCOLS,
+      META_PROP_VALUE_ATOM_LIST,
+      reload_wm_protocols,
+      LOAD_INIT
+    },
+    {
+      XA_WM_HINTS,
+      META_PROP_VALUE_WM_HINTS,
+      reload_wm_hints,
+      LOAD_INIT
+    },
+    {
+      display->atom__NET_WM_USER_TIME,
+      META_PROP_VALUE_CARDINAL,
+      reload_net_wm_user_time,
+      LOAD_INIT
+    },
+    {
+      display->atom__NET_WM_STATE,
+      META_PROP_VALUE_ATOM_LIST,
+      reload_net_wm_state,
+      LOAD_INIT | INIT_ONLY
+    },
+    {
+      display->atom__MOTIF_WM_HINTS,
+      META_PROP_VALUE_MOTIF_HINTS,
+      reload_mwm_hints,
+      LOAD_INIT
+    },
+    {
+      XA_WM_TRANSIENT_FOR,
+      META_PROP_VALUE_WINDOW,
+      reload_transient_for,
+      LOAD_INIT
+    },
+    {
+      display->atom__GTK_THEME_VARIANT,
+      META_PROP_VALUE_UTF8,
+      reload_gtk_theme_variant,
+      LOAD_INIT
+    },
+    {
+      display->atom__GTK_FRAME_EXTENTS,
+      META_PROP_VALUE_CARDINAL_LIST,
+      reload_gtk_frame_extents,
+      LOAD_INIT
+    },
+    {
+      display->atom__NET_WM_USER_TIME_WINDOW,
+      META_PROP_VALUE_WINDOW,
+      reload_net_wm_user_time_window,
+      LOAD_INIT
+    },
+    {
+      display->atom_WM_STATE,
+      META_PROP_VALUE_INVALID,
+      NULL,
+      NONE
+    },
+    {
+      display->atom__NET_WM_ICON,
+      META_PROP_VALUE_INVALID,
+      reload_net_wm_icon,
+      NONE
+    },
+    {
+      display->atom__KWM_WIN_ICON,
+      META_PROP_VALUE_INVALID,
+      reload_kwm_win_icon,
+      NONE
+    },
+    {
+      display->atom_WM_CLIENT_LEADER,
+      META_PROP_VALUE_INVALID,
+      complain_about_broken_client,
+      NONE
+    },
+    {
+      display->atom_SM_CLIENT_ID,
+      META_PROP_VALUE_INVALID,
+      complain_about_broken_client,
+      NONE
+    },
+    {
+      display->atom_WM_WINDOW_ROLE,
+      META_PROP_VALUE_STRING,
+      reload_wm_window_role,
+      LOAD_INIT | FORCE_INIT
+    },
+    {
+      display->atom__NET_WM_WINDOW_TYPE,
+      META_PROP_VALUE_ATOM_LIST,
+      reload_net_wm_window_type,
+      LOAD_INIT | FORCE_INIT
+    },
+    {
+      display->atom__NET_WM_STRUT,
+      META_PROP_VALUE_INVALID,
+      reload_struts,
+      NONE
+    },
+    {
+      display->atom__NET_WM_STRUT_PARTIAL,
+      META_PROP_VALUE_INVALID,
+      reload_struts,
+      NONE
+    },
+    {
+      0
+    },
   };
 
   MetaWindowPropHooks *table = g_memdup (hooks, sizeof (hooks)),
@@ -1525,6 +1853,12 @@ meta_display_init_window_prop_hooks (MetaDisplay *display)
 
   while (cursor->property)
     {
+      /* Doing initial loading doesn't make sense if we just want notification */
+      g_assert (!((cursor->flags & LOAD_INIT) && cursor->type == META_PROP_VALUE_INVALID));
+
+      /* Forcing initialization doesn't make sense if not loading initially */
+      g_assert ((cursor->flags & LOAD_INIT) || !(cursor->flags & FORCE_INIT));
+
       /* Atoms are safe to use with GINT_TO_POINTER because it's safe with
        * anything 32 bits or less, and atoms are 32 bits with the top three
        * bits clear.  (Scheifler & Gettys, 2e, p372)
@@ -1534,6 +1868,7 @@ meta_display_init_window_prop_hooks (MetaDisplay *display)
                            cursor);
       cursor++;
     }
+  display->n_prop_hooks = cursor - table;
 }
 
 void
@@ -1546,9 +1881,6 @@ meta_display_free_window_prop_hooks (MetaDisplay *display)
   display->prop_hooks_table = NULL;
 }
 
-/**
- * Finds the hooks for a particular property.
- */
 static MetaWindowPropHooks*
 find_hooks (MetaDisplay *display,
             Atom         property)
