@@ -277,9 +277,12 @@ meta_ui_new (Display *xdisplay,
 
   g_assert (xdisplay == GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()));
   ui->frames = meta_frames_new (XScreenNumberOfScreen (screen));
-  /* This does not actually show any widget. MetaFrames has been hacked so
-   * that showing it doesn't actually do anything. But we need the flags
-   * set for GTK to deliver events properly. */
+  /* GTK+ needs the frame-sync protocol to work in order to properly
+   * handle style changes. This means that the dummy widget we create
+   * to get the style for title bars actually needs to be mapped
+   * and fully tracked as a MetaWindow. Horrible, but mostly harmless -
+   * the window is a 1x1 overide redirect window positioned offscreen.
+   */
   gtk_widget_show (GTK_WIDGET (ui->frames));
 
   g_object_set_data (G_OBJECT (gdisplay), "meta-ui", ui);
@@ -300,6 +303,16 @@ meta_ui_free (MetaUI *ui)
   g_free (ui);
 }
 
+void
+meta_ui_get_frame_mask (MetaUI  *ui,
+                        Window   frame_xwindow,
+                        guint    width,
+                        guint    height,
+                        cairo_t *cr)
+{
+  meta_frames_get_mask (ui->frames, frame_xwindow, width, height, cr);
+}
+
 void
 meta_ui_get_frame_borders (MetaUI *ui,
                            Window frame_xwindow,
@@ -308,6 +321,17 @@ meta_ui_get_frame_borders (MetaUI *ui,
   meta_frames_get_borders (ui->frames, frame_xwindow, borders);
 }
 
+static void
+set_background_none (Display *xdisplay,
+                     Window   xwindow)
+{
+  XSetWindowAttributes attrs;
+
+  attrs.background_pixmap = None;
+  XChangeWindowAttributes (xdisplay, xwindow,
+                           CWBackPixmap, &attrs);
+}
+
 Window
 meta_ui_create_frame_window (MetaUI *ui,
                              Display *xdisplay,
@@ -367,6 +391,7 @@ meta_ui_create_frame_window (MetaUI *ui,
 		    &attrs, attributes_mask);
 
   gdk_window_resize (window, width, height);
+  set_background_none (xdisplay, GDK_WINDOW_XID (window));
 
   meta_frames_manage_window (ui->frames, GDK_WINDOW_XID (window), window);
 
@@ -419,16 +444,6 @@ meta_ui_unmap_frame (MetaUI *ui,
     gdk_window_hide (window);
 }
 
-void
-meta_ui_unflicker_frame_bg (MetaUI *ui,
-                            Window  xwindow,
-                            int     target_width,
-                            int     target_height)
-{
-  meta_frames_unflicker_bg (ui->frames, xwindow,
-                            target_width, target_height);
-}
-
 void
 meta_ui_update_frame_style (MetaUI *ui,
                             Window  xwindow)
@@ -443,13 +458,6 @@ meta_ui_repaint_frame (MetaUI *ui,
   meta_frames_repaint_frame (ui->frames, xwindow);
 }
 
-void
-meta_ui_reset_frame_bg (MetaUI *ui,
-                        Window xwindow)
-{
-  meta_frames_reset_bg (ui->frames, xwindow);
-}
-
 void
 meta_ui_apply_frame_shape  (MetaUI  *ui,
                             Window   xwindow,
@@ -462,6 +470,18 @@ meta_ui_apply_frame_shape  (MetaUI  *ui,
                             window_has_shape);
 }
 
+cairo_region_t *
+meta_ui_get_frame_bounds (MetaUI *ui,
+                          Window  xwindow,
+                          int     window_width,
+                          int     window_height)
+{
+  return meta_frames_get_frame_bounds (ui->frames,
+                                       xwindow,
+                                       window_width,
+                                       window_height);
+}
+
 void
 meta_ui_queue_frame_draw (MetaUI *ui,
                           Window xwindow)
@@ -574,6 +594,20 @@ meta_ui_pop_delay_exposes  (MetaUI *ui)
   meta_frames_pop_delay_exposes (ui->frames);
 }
 
+static GdkPixbuf *
+load_default_window_icon (int size)
+{
+  GtkIconTheme *theme = gtk_icon_theme_get_default ();
+  const char *icon_name;
+
+  if (gtk_icon_theme_has_icon (theme, META_DEFAULT_ICON_NAME))
+    icon_name = META_DEFAULT_ICON_NAME;
+  else
+    icon_name = "image-missing";
+
+  return gtk_icon_theme_load_icon (theme, icon_name, size, 0, NULL);
+}
+
 GdkPixbuf*
 meta_ui_get_default_window_icon (MetaUI *ui)
 {
@@ -581,26 +615,7 @@ meta_ui_get_default_window_icon (MetaUI *ui)
 
   if (default_icon == NULL)
     {
-      GtkIconTheme *theme;
-      gboolean icon_exists;
-
-      theme = gtk_icon_theme_get_default ();
-
-      icon_exists = gtk_icon_theme_has_icon (theme, META_DEFAULT_ICON_NAME);
-
-      if (icon_exists)
-          default_icon = gtk_icon_theme_load_icon (theme,
-                                                   META_DEFAULT_ICON_NAME,
-                                                   META_ICON_WIDTH,
-                                                   0,
-                                                   NULL);
-      else
-          default_icon = gtk_icon_theme_load_icon (theme,
-                                                   "image-missing",
-                                                   META_ICON_WIDTH,
-                                                   0,
-                                                   NULL);
-
+      default_icon = load_default_window_icon (META_ICON_WIDTH);
       g_assert (default_icon);
     }
 
@@ -616,26 +631,7 @@ meta_ui_get_default_mini_icon (MetaUI *ui)
 
   if (default_icon == NULL)
     {
-      GtkIconTheme *theme;
-      gboolean icon_exists;
-
-      theme = gtk_icon_theme_get_default ();
-
-      icon_exists = gtk_icon_theme_has_icon (theme, META_DEFAULT_ICON_NAME);
-
-      if (icon_exists)
-          default_icon = gtk_icon_theme_load_icon (theme,
-                                                   META_DEFAULT_ICON_NAME,
-                                                   META_MINI_ICON_WIDTH,
-                                                   0,
-                                                   NULL);
-      else
-          default_icon = gtk_icon_theme_load_icon (theme,
-                                                   "image-missing",
-                                                   META_MINI_ICON_WIDTH,
-                                                   0,
-                                                   NULL);
-
+      default_icon = load_default_window_icon (META_MINI_ICON_WIDTH);
       g_assert (default_icon);
     }
 
@@ -663,38 +659,6 @@ meta_ui_window_should_not_cause_focus (Display *xdisplay,
     return FALSE;
 }
 
-char*
-meta_text_property_to_utf8 (Display             *xdisplay,
-                            const XTextProperty *prop)
-{
-  GdkDisplay *display;
-  char **list;
-  int count;
-  char *retval;
-
-  list = NULL;
-
-  display = gdk_x11_lookup_xdisplay (xdisplay);
-  count = gdk_text_property_to_utf8_list_for_display (display,
-                                                      gdk_x11_xatom_to_atom_for_display (display, prop->encoding),
-                                                      prop->format,
-                                                      prop->value,
-                                                      prop->nitems,
-                                                      &list);
-
-  if (count == 0)
-    retval = NULL;
-  else
-    {
-      retval = list[0];
-      list[0] = g_strdup (""); /* something to free */
-    }
-
-  g_strfreev (list);
-
-  return retval;
-}
-
 void
 meta_ui_theme_get_frame_borders (MetaUI           *ui,
                                  MetaFrameType     type,
@@ -704,47 +668,68 @@ meta_ui_theme_get_frame_borders (MetaUI           *ui,
   int text_height;
   PangoContext *context;
   const PangoFontDescription *font_desc;
-  GtkStyleContext *style = NULL;
   PangoFontDescription *free_font_desc = NULL;
 
   if (meta_ui_have_a_theme ())
     {
+      GdkDisplay *display = gdk_x11_lookup_xdisplay (ui->xdisplay);
+      GdkScreen *screen = gdk_display_get_screen (display, XScreenNumberOfScreen (ui->xscreen));
+      MetaStyleInfo *style_info = NULL;
+      MetaTheme *current;
+
+      style_info = meta_theme_create_style_info (screen, NULL);
+
       context = gtk_widget_get_pango_context (GTK_WIDGET (ui->frames));
       font_desc = meta_prefs_get_titlebar_font ();
 
+      current = meta_theme_get_current ();
+
       if (!font_desc)
         {
-          GdkDisplay *display = gdk_x11_lookup_xdisplay (ui->xdisplay);
-          GdkScreen *screen = gdk_display_get_screen (display, XScreenNumberOfScreen (ui->xscreen));
-          GtkWidgetPath *widget_path;
+          if (current->is_gtk_theme == FALSE)
+            {
+              GtkStyleContext *style = NULL;
+              GtkWidgetPath *widget_path;
 
-          style = gtk_style_context_new ();
-          gtk_style_context_set_screen (style, screen);
-          widget_path = gtk_widget_path_new ();
-          gtk_widget_path_append_type (widget_path, GTK_TYPE_WINDOW);
-          gtk_style_context_set_path (style, widget_path);
-          gtk_widget_path_free (widget_path);
+              style = gtk_style_context_new ();
+              gtk_style_context_set_screen (style, screen);
+              widget_path = gtk_widget_path_new ();
+              gtk_widget_path_append_type (widget_path, GTK_TYPE_WINDOW);
+              gtk_style_context_set_path (style, widget_path);
+              gtk_widget_path_free (widget_path);
+
+              gtk_style_context_get (style, GTK_STATE_FLAG_NORMAL,
+                                     "font", &free_font_desc,
+                                     NULL);
+
+              if (style != NULL)
+                g_object_unref (style);
+            }
+          else
+            {
+              free_font_desc = meta_style_info_create_font_desc (style_info);
+            }
 
-          gtk_style_context_get (style, GTK_STATE_FLAG_NORMAL, "font", &free_font_desc, NULL);
           font_desc = (const PangoFontDescription *) free_font_desc;
         }
 
       text_height = meta_pango_font_desc_get_text_height (font_desc, context);
 
-      meta_theme_get_frame_borders (meta_theme_get_current (),
+      meta_theme_get_frame_borders (current,
+                                    style_info,
                                     type, text_height, flags,
                                     borders);
 
       if (free_font_desc)
         pango_font_description_free (free_font_desc);
+
+      if (style_info != NULL)
+        meta_style_info_unref (style_info);
     }
   else
     {
       meta_frame_borders_clear (borders);
     }
-
-  if (style != NULL)
-    g_object_unref (style);
 }
 
 void
@@ -867,43 +852,6 @@ meta_ui_parse_accelerator (const char          *accel,
   return TRUE;
 }
 
-/* Caller responsible for freeing return string of meta_ui_accelerator_name! */
-gchar*
-meta_ui_accelerator_name  (unsigned int        keysym,
-                           MetaVirtualModifier mask)
-{
-  GdkModifierType mods = 0;
-
-  if (keysym == 0 && mask == 0)
-    {
-      return g_strdup ("disabled");
-    }
-
-  if (mask & META_VIRTUAL_SHIFT_MASK)
-    mods |= GDK_SHIFT_MASK;
-  if (mask & META_VIRTUAL_CONTROL_MASK)
-    mods |= GDK_CONTROL_MASK;
-  if (mask & META_VIRTUAL_ALT_MASK)
-    mods |= GDK_MOD1_MASK;
-  if (mask & META_VIRTUAL_MOD2_MASK)
-    mods |= GDK_MOD2_MASK;
-  if (mask & META_VIRTUAL_MOD3_MASK)
-    mods |= GDK_MOD3_MASK;
-  if (mask & META_VIRTUAL_MOD4_MASK)
-    mods |= GDK_MOD4_MASK;
-  if (mask & META_VIRTUAL_MOD5_MASK)
-    mods |= GDK_MOD5_MASK;
-  if (mask & META_VIRTUAL_SUPER_MASK)
-    mods |= GDK_SUPER_MASK;
-  if (mask & META_VIRTUAL_HYPER_MASK)
-    mods |= GDK_HYPER_MASK;
-  if (mask & META_VIRTUAL_META_MASK)
-    mods |= GDK_META_MASK;
-
-  return gtk_accelerator_name (keysym, mods);
-
-}
-
 gboolean
 meta_ui_parse_modifier (const char          *accel,
                         MetaVirtualModifier *mask)
@@ -993,3 +941,30 @@ meta_ui_get_direction (void)
 
   return META_UI_DIRECTION_LTR;
 }
+
+GdkPixbuf *
+meta_ui_get_pixbuf_from_pixmap (Pixmap pixmap)
+{
+  Display *display;
+  Window root;
+  int x, y;
+  unsigned int width, height, border, depth;
+  GdkVisual *visual;
+  cairo_surface_t *surface;
+  GdkPixbuf *pixbuf;
+
+  display = GDK_DISPLAY_XDISPLAY (gdk_display_get_default ());
+
+  if (!XGetGeometry (display, pixmap, &root, &x, &y, &width, &height, &border, &depth))
+    return NULL;
+
+  visual = gdk_screen_get_rgba_visual (gdk_screen_get_default());
+  if (!visual)
+    visual = gdk_screen_get_system_visual (gdk_screen_get_default());
+
+  surface = cairo_xlib_surface_create (display, pixmap, GDK_VISUAL_XVISUAL (visual), width, height);
+  pixbuf = gdk_pixbuf_get_from_surface (surface, x, y, width, height);
+  cairo_surface_destroy (surface);
+
+  return pixbuf;
+}
